@@ -402,6 +402,59 @@ int mutt_matches_ignore (const char *s, LIST *t)
   return 0;
 }
 
+/* Splits src into parts delimited by delimiter.
+ * Invokes mapfunc on each part and joins the result back into src.
+ * Note this function currently does not preserve trailing delimiters.
+ */
+static void delimited_buffer_map_join (BUFFER *src, const char *delimiter,
+                                       void (*mapfunc)(BUFFER *))
+{
+  BUFFER *dest, *part;
+  const char *part_begin, *part_end;
+  size_t delim_size;
+
+  delim_size = mutt_strlen (delimiter);
+  if (!delim_size)
+  {
+    mapfunc (src);
+    return;
+  }
+
+  dest = mutt_buffer_pool_get ();
+  part = mutt_buffer_pool_get ();
+
+  part_begin = mutt_b2s (src);
+  while (part_begin && *part_begin)
+  {
+    part_end = strstr (part_begin, delimiter);
+    if (part_end)
+    {
+      mutt_buffer_substrcpy (part, part_begin, part_end);
+      part_end += delim_size;
+    }
+    else
+      mutt_buffer_strcpy (part, part_begin);
+
+    mapfunc (part);
+
+    if (part_begin != mutt_b2s (src))
+      mutt_buffer_addstr (dest, delimiter);
+    mutt_buffer_addstr (dest, mutt_b2s (part));
+
+    part_begin = part_end;
+  }
+
+  mutt_buffer_strcpy (src, mutt_b2s (dest));
+
+  mutt_buffer_pool_release (&dest);
+  mutt_buffer_pool_release (&part);
+}
+
+void mutt_buffer_expand_multi_path (BUFFER *src, const char *delimiter)
+{
+  delimited_buffer_map_join (src, delimiter, mutt_buffer_expand_path);
+}
+
 char *mutt_expand_path (char *s, size_t slen)
 {
   return _mutt_expand_path (s, slen, 0);
@@ -920,6 +973,11 @@ void mutt_free_alias (ALIAS **p)
   }
 }
 
+void mutt_buffer_pretty_multi_mailbox (BUFFER *s, const char *delimiter)
+{
+  delimited_buffer_map_join (s, delimiter, mutt_buffer_pretty_mailbox);
+}
+
 void mutt_buffer_pretty_mailbox (BUFFER *s)
 {
   /* This reduces the size of the BUFFER, so we can pass it through.
@@ -1081,6 +1139,52 @@ void mutt_buffer_sanitize_filename (BUFFER *d, const char *f, short slash)
   }
 }
 
+static int is_ansi (const char *buf)
+{
+  while (*buf && (isdigit(*buf) || *buf == ';'))
+    buf++;
+  return (*buf == 'm');
+}
+
+/* Removes ANSI and backspace formatting.
+ *
+ * This logic is pulled from the pager fill_buffer() function, for use
+ * in stripping reply-quoted autoview output of ansi sequences.
+ */
+void mutt_buffer_strip_formatting (BUFFER *dest, const char *src)
+{
+  const char *s = src;
+
+  mutt_buffer_clear (dest);
+
+  if (!s)
+    return;
+
+  while (*s)
+  {
+    if (*s == '\010' && (s > src))
+    {
+      if (*(s+1) == '_')	/* underline */
+        s += 2;
+      else if (*(s+1) && mutt_buffer_len (dest))	/* bold or overstrike */
+      {
+        dest->dptr--;
+        mutt_buffer_addch (dest, *(s+1));
+        s += 2;
+      }
+      else			/* ^H */
+        mutt_buffer_addch (dest, *s++);
+    }
+    else if (*s == '\033' && *(s+1) == '[' && is_ansi (s + 2))
+    {
+      while (*s++ != 'm')	/* skip ANSI sequence */
+        ;
+    }
+    else
+      mutt_buffer_addch (dest, *s++);
+  }
+}
+
 void mutt_expand_file_fmt (BUFFER *dest, const char *fmt, const char *src)
 {
   BUFFER *tmp;
@@ -1400,7 +1504,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
                         int cols,               /* maximum columns */
 			const char *src,	/* template string */
 			format_t *callback,	/* callback for processing */
-			unsigned long data,	/* callback data */
+			void *data,		/* callback data */
 			format_flag flags)	/* callback flags */
 {
   char prefix[SHORT_STRING], buf[LONG_STRING], *cp, *wptr = dest, ch;
