@@ -280,7 +280,7 @@ static int pgp_check_pgp_decryption_okay_regexp (FILE *fpin)
  *   1 - no patterns were matched (if delegated to decryption_okay_regexp)
  *   0 - DECRYPTION_OKAY was seen, with no PLAINTEXT outside.
  *  -1 - No decryption status codes were encountered
- *  -2 - PLAINTEXT was encountered outside of DECRYPTION delimeters.
+ *  -2 - PLAINTEXT was encountered outside of DECRYPTION delimiters.
  *  -3 - DECRYPTION_FAILED was encountered
  */
 static int pgp_check_decryption_okay (FILE *fpin)
@@ -1022,7 +1022,6 @@ BODY *pgp_decrypt_part (BODY *a, STATE *s, FILE *fpout, BODY *p)
       p->goodsig = 1;
     else
       p->goodsig = 0;
-    state_attach_puts (_("[-- End of PGP output --]\n\n"), s);
   }
   safe_fclose (&pgperr);
 
@@ -1153,7 +1152,7 @@ int pgp_encrypted_handler (BODY *a, STATE *s)
   BUFFER *tempfile = NULL;
   FILE *fpout, *fpin;
   BODY *tattach;
-  int rc = -1;
+  int rc = 1;
 
   tempfile = mutt_buffer_pool_get ();
   mutt_buffer_mktemp (tempfile);
@@ -1164,9 +1163,15 @@ int pgp_encrypted_handler (BODY *a, STATE *s)
     goto cleanup;
   }
 
-  if (s->flags & MUTT_DISPLAY) crypt_current_time (s, "PGP");
+  if (s->flags & MUTT_DISPLAY)
+    crypt_current_time (s, "PGP");
 
-  if ((tattach = pgp_decrypt_part (a, s, fpout, a)) != NULL)
+  tattach = pgp_decrypt_part (a, s, fpout, a);
+
+  if (s->flags & MUTT_DISPLAY)
+    state_attach_puts (_("[-- End of PGP output --]\n\n"), s);
+
+  if (tattach != NULL)
   {
     if (s->flags & MUTT_DISPLAY)
     {
@@ -1222,8 +1227,9 @@ int pgp_encrypted_handler (BODY *a, STATE *s)
   }
   else
   {
-    mutt_error _("Could not decrypt PGP message");
-    mutt_sleep (2);
+    if (s->flags & MUTT_DISPLAY)
+      state_attach_puts (_("[-- Error: decryption failed --]\n\n"), s);
+
     /* void the passphrase, even if it's not necessarily the problem */
     pgp_void_passphrase ();
   }
@@ -1797,16 +1803,19 @@ cleanup:
   return b;
 }
 
-int pgp_send_menu (HEADER *msg)
+void pgp_send_menu (SEND_CONTEXT *sctx)
 {
+  HEADER *msg;
   pgp_key_t p;
   char input_signas[SHORT_STRING];
   char *prompt, *letters, *choices;
   char promptbuf[LONG_STRING];
   int choice;
 
+  msg = sctx->msg;
+
   if (!(WithCrypto & APPLICATION_PGP))
-    return msg->security;
+    return;
 
   /* If autoinline and no crypto options set, then set inline. */
   if (option (OPTPGPAUTOINLINE) &&
@@ -1918,7 +1927,7 @@ int pgp_send_menu (HEADER *msg)
         {
           snprintf (input_signas, sizeof (input_signas), "0x%s",
                     pgp_fpr_or_lkeyid (p));
-          mutt_str_replace (&PgpSignAs, input_signas);
+          mutt_str_replace (&sctx->pgp_sign_as, input_signas);
           pgp_free_key (&p);
 
           msg->security |= SIGN;
@@ -1955,8 +1964,6 @@ int pgp_send_menu (HEADER *msg)
         break;
     }
   }
-
-  return (msg->security);
 }
 
 
