@@ -1,20 +1,20 @@
 /*
  * Copyright (C) 2000-2002,2004 Thomas Roessler <roessler@does-not-exist.org>
- * 
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */ 
+ */
 
 /*
  * A simple URL parser.
@@ -67,7 +67,8 @@ static int url_pct_decode (char *s)
       }
       else
 	return -1;
-    } else
+    }
+    else
       *d++ = *s;
   }
   *d ='\0';
@@ -213,21 +214,34 @@ static void url_pct_encode (char *dst, size_t l, const char *src)
   *dst = 0;
 }
 
-/* url_ciss_tostring: output the URL string for a given CISS object. */
 int url_ciss_tostring (ciss_url_t* ciss, char* dest, size_t len, int flags)
 {
-  long l;
+  BUFFER *dest_buf;
+  int retval;
 
+  dest_buf = mutt_buffer_pool_get ();
+
+  retval = url_ciss_tobuffer (ciss, dest_buf, flags);
+  if (!retval)
+    strfcpy (dest, mutt_b2s (dest_buf), len);
+
+  mutt_buffer_pool_release (&dest_buf);
+
+  return retval;
+}
+
+/* url_ciss_tobuffer: output the URL string for a given CISS object. */
+int url_ciss_tobuffer (ciss_url_t* ciss, BUFFER* dest, int flags)
+{
   if (ciss->scheme == U_UNKNOWN)
     return -1;
 
-  snprintf (dest, len, "%s:", mutt_getnamebyvalue (ciss->scheme, UrlMap));
+  mutt_buffer_printf (dest, "%s:", mutt_getnamebyvalue (ciss->scheme, UrlMap));
 
   if (ciss->host)
   {
     if (!(flags & U_PATH))
-      safe_strcat (dest, len, "//");
-    len -= (l = strlen (dest)); dest += l;
+      mutt_buffer_addstr (dest, "//");
 
     if (ciss->user)
     {
@@ -238,29 +252,25 @@ int url_ciss_tostring (ciss_url_t* ciss, char* dest, size_t len, int flags)
       {
 	char p[STRING];
 	url_pct_encode (p, sizeof (p), ciss->pass);
-	snprintf (dest, len, "%s:%s@", u, p);
+	mutt_buffer_add_printf (dest, "%s:%s@", u, p);
       }
       else
-	snprintf (dest, len, "%s@", u);
-
-      len -= (l = strlen (dest)); dest += l;
+	mutt_buffer_add_printf (dest, "%s@", u);
     }
 
     if (strchr (ciss->host, ':'))
-      snprintf (dest, len, "[%s]", ciss->host);
+      mutt_buffer_add_printf (dest, "[%s]", ciss->host);
     else
-      snprintf (dest, len, "%s", ciss->host);
-
-    len -= (l = strlen (dest)); dest += l;
+      mutt_buffer_add_printf (dest, "%s", ciss->host);
 
     if (ciss->port)
-      snprintf (dest, len, ":%hu/", ciss->port);
+      mutt_buffer_add_printf (dest, ":%hu/", ciss->port);
     else
-      snprintf (dest, len, "/");
+      mutt_buffer_addstr (dest, "/");
   }
 
   if (ciss->path)
-    safe_strcat (dest, len, ciss->path);
+    mutt_buffer_addstr (dest, ciss->path);
 
   return 0;
 }
@@ -338,16 +348,7 @@ int url_parse_mailto (ENVELOPE *e, char **body, const char *src)
   }
 
   /* RFC2047 decode after the RFC822 parsing */
-  rfc2047_decode_adrlist (e->from);
-  rfc2047_decode_adrlist (e->to);
-  rfc2047_decode_adrlist (e->cc);
-  rfc2047_decode_adrlist (e->bcc);
-  rfc2047_decode_adrlist (e->reply_to);
-  rfc2047_decode_adrlist (e->mail_followup_to);
-  rfc2047_decode_adrlist (e->return_path);
-  rfc2047_decode_adrlist (e->sender);
-  rfc2047_decode (&e->x_label);
-  rfc2047_decode (&e->subject);
+  rfc2047_decode_envelope (e);
 
   rc = 0;
 
@@ -355,4 +356,3 @@ out:
   FREE (&tmp);
   return rc;
 }
-
